@@ -4,12 +4,13 @@ use Carp;
 use IO::File;
 use XML::Parser;
 use Data::Dumper;
+use Encode;
 
 use strict;
 use warnings;
 
 our $AUTOLOAD;
-our $VERSION = 0.5;
+our $VERSION = 0.6;
 
 my %component_attr_values = (
 	LocalOnly                 => 0,
@@ -95,6 +96,7 @@ sub new {
 		foreign_keys	=> {}, # tables referenced by foreign keys
 		correct_case	=> {}, # stores case-sensitive table names
 		filename	=> undef,
+		encoding        => undef,
 	};
 
 	bless $self, $class;
@@ -180,6 +182,35 @@ sub _openfile {
 
 }
 
+=item I<set_encoding>
+
+  $is->set_encoding('');
+
+Sets the encoding that will be used when writing tables that
+have been modified. By default, the encoding will be whatever
+appears in the XML declaration at the beginning of the ism.
+If none appears, no encoding will be done.
+
+=cut
+sub set_encoding {
+	my ($self, $encoding) = @_;
+	$self->{'encoding'} = $encoding;
+}
+
+=item I<get_encoding>
+
+  my $encoding = $is->get_encoding();
+
+Returns the encoding that will be used when writing tables
+that have been modified. Returns undef if the encoding is
+unknown, in which case no encoding will be done.
+
+=cut
+sub get_encoding {
+	my ($self) = @_;
+	return $self->{'encoding'};
+}
+
 =item I<loadfile>
 
   $is->loadfile( $filename );
@@ -227,6 +258,7 @@ sub load {
 	$self->{'foreign_keys'} = {};
 	$self->{'correct_case'} = {};
 	$self->{'filename'} = undef;
+	$self->{'encoding'} = undef;
 
 	my $section = 'header';
 	my $lastsection = $section;
@@ -270,6 +302,14 @@ sub load {
 		push(@{$self->{'sections'}{$section}}, $_);
 	}
 
+	# find the file encoding
+	foreach my $line (@{$self->{sections}->{header}}) {
+		if($line =~ /\sencoding="([^"]+)"/) {
+			$self->{'encoding'} = $1;
+			last;
+		}
+	}
+
 	return 1;
 }
 
@@ -319,6 +359,9 @@ Returns the ism data as a string.
 =cut
 sub save {
 	my ($self) = @_;
+	
+	my $encoding = $self->{'encoding'};
+	my $has_encoding = defined($encoding);
 
 	my $text = '';
 
@@ -326,10 +369,15 @@ sub save {
 		if($self->{'parsed'}{$section}) {
 			# the table has been (possibly) modified, so rebuild it
 			if($section eq 'summary') {
-				$text .= $self->_save_summary;
+				$text .= ($has_encoding) ?
+					encode($encoding, $self->_save_summary) :
+					$self->_save_summary;
 			} else {
-				$text .= $self->_save_table($section);
+				$text .= ($has_encoding) ?
+					encode($encoding, $self->_save_table($section)) :
+					$self->_save_table($section);
 			}
+
 		} else {
 			# when the last table gets modified, we end up with an
 			# extra newline
@@ -340,7 +388,6 @@ sub save {
 			$text .= join("\n", @{$self->{'sections'}{$section}}) . "\n";
 		}
 	}
-
 	return $text;
 }
 
